@@ -1,7 +1,6 @@
 pub(crate) mod config;
 mod participation;
 mod round_success_meter;
-mod synchronizer;
 #[cfg(test)]
 mod tests;
 
@@ -17,7 +16,7 @@ use datasize::DataSize;
 use itertools::Itertools;
 use num_traits::AsPrimitive;
 use serde::{Deserialize, Serialize};
-use tracing::{error, info, trace, warn};
+use tracing::{debug, error, info, trace, warn};
 
 use casper_types::{system::auction::BLOCK_REWARD, U512};
 
@@ -36,6 +35,7 @@ use crate::{
             },
             state,
             state::{Observation, Panorama},
+            synchronizer::Synchronizer,
             validators::{ValidatorIndex, Validators},
         },
         traits::{ConsensusValueT, Context, NodeIdT},
@@ -45,7 +45,7 @@ use crate::{
 };
 
 pub use self::config::Config as HighwayConfig;
-use self::{round_success_meter::RoundSuccessMeter, synchronizer::Synchronizer};
+use self::round_success_meter::RoundSuccessMeter;
 
 /// Never allow more than this many units in a piece of evidence for conflicting endorsements,
 /// even if eras are longer than this.
@@ -67,7 +67,7 @@ const TIMER_ID_SYNCHRONIZER_LOG: TimerId = TimerId(5);
 const TIMER_ID_PROGRESS_ALERT: TimerId = TimerId(6);
 
 /// The action of adding a vertex from the `vertices_to_be_added` queue.
-const ACTION_ID_VERTEX: ActionId = ActionId(0);
+pub(crate) const ACTION_ID_VERTEX: ActionId = ActionId(0);
 
 #[derive(DataSize, Debug)]
 pub(crate) struct HighwayProtocol<I, C>
@@ -87,9 +87,12 @@ where
     /// The panorama snapshot. This is updated periodically, and if it does not change for too
     /// long, an alert is raised.
     last_panorama: Panorama<C>,
-    /// If the current era's protocol state has not progressed for this long, return
-    /// `ProtocolOutcome::StandstillAlert`.
+    /// If the current era's protocol state has not progressed for this long, request the latest
+    /// state from peers.
     standstill_timeout: TimeDiff,
+    /// If after another `standstill_timeout` there is no progress, raise
+    /// `ProtocolOutcome::StandstillAlert` and shut down.
+    shutdown_on_standstill: bool,
     /// Log inactive or faulty validators periodically, with this interval.
     log_participation_interval: TimeDiff,
     /// Whether to log the size of every incoming and outgoing serialized unit.
@@ -102,7 +105,8 @@ impl<I: NodeIdT, C: Context + 'static> HighwayProtocol<I, C> {
     pub(crate) fn new_boxed(
         instance_id: C::InstanceId,
         validator_stakes: BTreeMap<C::ValidatorId, U512>,
-        slashed: &HashSet<C::ValidatorId>,
+        faulty: &HashSet<C::ValidatorId>,
+        inactive: &HashSet<C::ValidatorId>,
         protocol_config: &ProtocolConfig,
         config: &Config,
         prev_cp: Option<&dyn ConsensusProtocol<I, C>>,
@@ -125,9 +129,17 @@ impl<I: NodeIdT, C: Context + 'static> HighwayProtocol<I, C> {
         let mut validators: Validators<C::ValidatorId> =
             validator_stakes.into_iter().map(scale_stake).collect();
 
-        for vid in slashed {
+        for vid in faulty {
             validators.ban(vid);
         }
+        for vid in inactive {
+            validators.set_cannot_propose(vid);
+        }
+
+        assert!(
+            validators.ensure_nonzero_proposing_stake(),
+            "cannot start era with total weight 0"
+        );
 
         let highway_config = &protocol_config.highway_config;
 
@@ -164,7 +176,7 @@ impl<I: NodeIdT, C: Context + 'static> HighwayProtocol<I, C> {
 
         // Allow about as many units as part of evidence for conflicting endorsements as we expect
         // a validator to create during an era. After that, they can endorse two conflicting forks
-        // without getting slashed.
+        // without getting faulty.
         let min_round_len = state::round_len(highway_config.minimum_round_exponent);
         let min_rounds_per_era = protocol_config
             .minimum_era_height
@@ -200,6 +212,7 @@ impl<I: NodeIdT, C: Context + 'static> HighwayProtocol<I, C> {
             evidence_only: false,
             last_panorama,
             standstill_timeout: config.highway.standstill_timeout,
+            shutdown_on_standstill: config.highway.shutdown_on_standstill,
             log_participation_interval: config.highway.log_participation_interval,
             log_unit_sizes: config.highway.log_unit_sizes,
         });
@@ -465,17 +478,36 @@ impl<I: NodeIdT, C: Context + 'static> HighwayProtocol<I, C> {
             return vec![]; // Era has ended. No further progress is expected.
         }
         if self.last_panorama == *self.highway.state().panorama() {
+            info!(
+                instance_id = ?self.highway.instance_id(),
+                "no progress in the last {}, creating latest state request",
+                self.standstill_timeout,
+            );
             // We haven't made any progress. Request latest panorama from peers and schedule
             // standstill alert. If we still won't progress by the time
             // `TIMER_ID_STANDSTILL_ALERT` is handled, it means we're stuck.
             let mut outcomes = self.latest_panorama_request();
-            outcomes.push(ProtocolOutcome::ScheduleTimer(
-                now + self.standstill_timeout,
-                TIMER_ID_STANDSTILL_ALERT,
-            ));
+            if self.shutdown_on_standstill {
+                outcomes.push(ProtocolOutcome::ScheduleTimer(
+                    now + self.standstill_timeout,
+                    TIMER_ID_STANDSTILL_ALERT,
+                ));
+            }
             return outcomes;
         }
 
+        if !self.shutdown_on_standstill {
+            debug!(
+                instance_id = ?self.highway.instance_id(),
+                "progress detected; not requesting latest state",
+            );
+            return vec![];
+        }
+        debug!(
+            instance_id = ?self.highway.instance_id(),
+            "progress detected; scheduling next standstill check in {}",
+            self.standstill_timeout,
+        );
         // Record the current panorama and schedule the next standstill check.
         self.last_panorama = self.highway.state().panorama().clone();
         vec![ProtocolOutcome::ScheduleTimer(
@@ -486,12 +518,24 @@ impl<I: NodeIdT, C: Context + 'static> HighwayProtocol<I, C> {
 
     /// Returns a `StandstillAlert` if no progress was made; otherwise schedules the next check.
     fn handle_standstill_alert_timer(&mut self, now: Timestamp) -> ProtocolOutcomes<I, C> {
-        if self.evidence_only || self.finalized_switch_block() {
-            return vec![]; // Era has ended. No further progress is expected.
+        if self.evidence_only || self.finalized_switch_block() || !self.shutdown_on_standstill {
+            // Era has ended and no further progress is expected, or shutdown on standstill is
+            // turned off.
+            return vec![];
         }
         if self.last_panorama == *self.highway.state().panorama() {
+            info!(
+                instance_id = ?self.highway.instance_id(),
+                "no progress in the last {}, raising standstill alert",
+                self.standstill_timeout,
+            );
             return vec![ProtocolOutcome::StandstillAlert]; // No progress within the timeout.
         }
+        debug!(
+            instance_id = ?self.highway.instance_id(),
+            "progress detected; scheduling next standstill check in {}",
+            self.standstill_timeout,
+        );
         // Record the current panorama and schedule the next standstill check.
         self.last_panorama = self.highway.state().panorama().clone();
         vec![ProtocolOutcome::ScheduleTimer(
@@ -540,7 +584,6 @@ impl<I: NodeIdT, C: Context + 'static> HighwayProtocol<I, C> {
 
     /// Creates a message to be gossiped that sends the validator's panorama.
     fn latest_panorama_request(&self) -> ProtocolOutcomes<I, C> {
-        trace!(instance_id=?self.highway.instance_id(), "creating latest state request");
         let request = HighwayMessage::LatestStateRequest(self.highway.state().panorama().clone());
         vec![ProtocolOutcome::CreatedGossipMessage(
             (&request).serialize(),
@@ -553,14 +596,14 @@ impl<I: NodeIdT, C: Context + 'static> HighwayProtocol<I, C> {
     serialize = "C::Hash: Serialize",
     deserialize = "C::Hash: Deserialize<'de>",
 ))]
-enum HighwayMessage<C: Context> {
+pub(crate) enum HighwayMessage<C: Context> {
     NewVertex(Vertex<C>),
     RequestDependency(Dependency<C>),
     LatestStateRequest(Panorama<C>),
 }
 
 impl<C: Context> HighwayMessage<C> {
-    fn serialize(&self) -> Vec<u8> {
+    pub(crate) fn serialize(&self) -> Vec<u8> {
         bincode::serialize(self).expect("should serialize message")
     }
 }
@@ -806,7 +849,7 @@ where
             outcomes.extend(self.detect_finality());
             outcomes
         } else {
-            // TODO: Slash proposer?
+            // TODO: Report proposer as faulty?
             // Drop vertices dependent on the invalid value.
             let dropped_vertices = self.pending_values.remove(&proposed_block);
             warn!(?proposed_block, ?dropped_vertices, "proposal is invalid");
