@@ -222,7 +222,7 @@ impl<T> Serialize for Responder<T> {
 
 /// Effect extension for futures, used to convert futures into actual effects.
 pub trait EffectExt: Future + Send {
-    /// Finalizes a future into an effect that returns an event.
+    /// Finalizes a future into an effect that returns a single event.
     ///
     /// The function `f` is used to translate the returned value from an effect into an event.
     fn event<U, F>(self, f: F) -> Effects<U>
@@ -231,6 +231,17 @@ pub trait EffectExt: Future + Send {
         U: 'static,
         Self: Sized;
 
+    /// Finalizes a future into an effect that returns an iterator of events.
+    ///
+    /// The function `f` is used to translate the returned value from an effect into an iterator of
+    /// events.
+    fn events<U, F, I>(self, f: F) -> Effects<U>
+    where
+        F: FnOnce(Self::Output) -> I + 'static + Send,
+        U: 'static,
+        I: Iterator<Item = U>,
+        Self: Sized;
+
     /// Finalizes a future into an effect that runs but drops the result.
     fn ignore<Ev>(self) -> Effects<Ev>;
 }
@@ -292,6 +303,15 @@ where
         smallvec![self.map(f).map(|item| smallvec![item]).boxed()]
     }
 
+    fn events<U, F, I>(self, f: F) -> Effects<U>
+    where
+        F: FnOnce(Self::Output) -> I + 'static + Send,
+        U: 'static,
+        I: Iterator<Item = U>,
+    {
+        smallvec![self.map(f).map(|iter| iter.collect()).boxed()]
+    }
+
     fn ignore<Ev>(self) -> Effects<Ev> {
         smallvec![self.map(|_| Multiple::new()).boxed()]
     }
@@ -653,6 +673,20 @@ impl<REv> EffectBuilder<REv> {
         )
     }
 
+    /// Announces that we have finished gossiping the indicated item.
+    pub(crate) async fn announce_finished_gossiping<T>(self, item_id: T::Id)
+    where
+        REv: From<GossiperAnnouncement<T>>,
+        T: Item,
+    {
+        self.0
+            .schedule(
+                GossiperAnnouncement::FinishedGossiping(item_id),
+                QueueKind::Regular,
+            )
+            .await;
+    }
+
     /// Announces that an invalid deploy has been received.
     pub(crate) fn announce_invalid_deploy<I>(
         self,
@@ -1602,9 +1636,27 @@ impl<REv> EffectBuilder<REv> {
                 .and_then(|era_validators| era_validators.get(&era_id).cloned())
         } else {
             // in other eras, we just use the validators from the key block
-            self.get_key_block_header_for_era_id_from_storage(era_id)
+            let key_block_result = self
+                .get_key_block_header_for_era_id_from_storage(era_id)
                 .await
-                .and_then(|kb_hdr| kb_hdr.next_era_validator_weights().cloned())
+                .and_then(|kb_hdr| kb_hdr.next_era_validator_weights().cloned());
+            if key_block_result.is_some() {
+                // if the key block read was successful, just return it
+                key_block_result
+            } else {
+                // if there was no key block, we might be looking at a future era - in such a case,
+                // read the state root hash from the highest block and check with the contract
+                // runtime
+                let highest_block = self.get_highest_block_from_storage().await?;
+                let req = EraValidatorsRequest::new(
+                    (*highest_block.header().state_root_hash()).into(),
+                    protocol_version,
+                );
+                self.get_era_validators_from_contract_runtime(req)
+                    .await
+                    .ok()
+                    .and_then(|era_validators| era_validators.get(&era_id).cloned())
+            }
         }
     }
 
